@@ -44,18 +44,23 @@ class SaltCMD(parsers.SaltCMDOptionParser):
             # process is run with the -a flag
             skip_perm_errors = self.options.eauth != ''
 
-            local = salt.client.get_local_client(
+            self.local_client = salt.client.get_local_client(
                 self.get_config_file_path(),
-                skip_perm_errors=skip_perm_errors)
+                skip_perm_errors=skip_perm_errors,
+                auto_reconnect=True)
         except SaltClientError as exc:
             self.exit(2, '{0}\n'.format(exc))
             return
 
         if self.options.batch or self.options.static:
+            # _run_batch() will handle all output and
+            # exit with the appropriate error condition
+            # Execution will not continue past this point
+            # in batch mode.
             self._run_batch()
         else:
             if self.options.timeout <= 0:
-                self.options.timeout = local.opts['timeout']
+                self.options.timeout = self.local_client.opts['timeout']
 
             kwargs = {
                 'tgt': self.config['tgt'],
@@ -115,20 +120,20 @@ class SaltCMD(parsers.SaltCMDOptionParser):
                 kwargs['eauth'] = self.options.eauth
 
             if self.config['async']:
-                jid = local.cmd_async(**kwargs)
+                jid = self.local_client.cmd_async(**kwargs)
                 print_cli('Executed command with job ID: {0}'.format(jid))
                 return
             retcodes = []
             try:
                 # local will be None when there was an error
                 errors = []
-                if local:
+                if self.local_client:
                     if self.options.subset:
-                        cmd_func = local.cmd_subset
+                        cmd_func = self.local_client.cmd_subset
                         kwargs['sub'] = self.options.subset
                         kwargs['cli'] = True
                     else:
-                        cmd_func = local.cmd_cli
+                        cmd_func = self.local_client.cmd_cli
 
                     if self.options.progress:
                         kwargs['progress'] = True
@@ -147,7 +152,7 @@ class SaltCMD(parsers.SaltCMDOptionParser):
                     elif self.config['fun'] == 'sys.doc':
                         ret = {}
                         out = ''
-                        for full_ret in local.cmd_cli(**kwargs):
+                        for full_ret in self.local_client.cmd_cli(**kwargs):
                             ret_, out, retcode = self._format_ret(full_ret)
                             ret.update(ret_)
                         self._output_ret(ret, out)
@@ -183,6 +188,7 @@ class SaltCMD(parsers.SaltCMDOptionParser):
                 ret = str(exc)
                 out = ''
                 self._output_ret(ret, out)
+                sys.exit(11)
 
     def _run_batch(self):
         import salt.cli.batch
@@ -213,7 +219,10 @@ class SaltCMD(parsers.SaltCMDOptionParser):
             if not self.options.batch:
                 self.config['batch'] = '100%'
 
-            batch = salt.cli.batch.Batch(self.config, eauth=eauth, quiet=True)
+            try:
+                batch = salt.cli.batch.Batch(self.config, eauth=eauth, quiet=True)
+            except salt.exceptions.SaltClientError as exc:
+                sys.exit(2)
 
             ret = {}
 
@@ -229,10 +238,14 @@ class SaltCMD(parsers.SaltCMDOptionParser):
                 # We will print errors to the console further down the stack
                 sys.exit(1)
             # Printing the output is already taken care of in run() itself
+            retcode = 0
             for res in batch.run():
-                if self.options.failhard:
-                    for ret in six.itervalues(res):
-                        retcode = salt.utils.job.get_retcode(ret)
+                for ret in six.itervalues(res):
+                    job_retcode = salt.utils.job.get_retcode(ret)
+                    if job_retcode > retcode:
+                        # Exit with the highest retcode we find
+                        retcode = job_retcode
+                    if self.options.failhard:
                         if retcode != 0:
                             sys.stderr.write(
                                 '{0}\nERROR: Minions returned with non-zero exit code.\n'.format(
@@ -240,6 +253,7 @@ class SaltCMD(parsers.SaltCMDOptionParser):
                                 )
                             )
                             sys.exit(retcode)
+            sys.exit(retcode)
 
     def _print_errors_summary(self, errors):
         if errors:
@@ -247,8 +261,8 @@ class SaltCMD(parsers.SaltCMDOptionParser):
             print_cli('---------------------------')
             print_cli('Errors')
             print_cli('---------------------------')
-            for minion in errors:
-                print_cli(self._format_error(minion))
+            for error in errors:
+                print_cli(self._format_error(error))
 
     def _print_returns_summary(self, ret):
         '''
@@ -358,13 +372,16 @@ class SaltCMD(parsers.SaltCMDOptionParser):
         if isinstance(ret, str):
             self.exit(2, '{0}\n'.format(ret))
         for host in ret:
-            if ret[host] == 'Minion did not return. [Not connected]':
+            if isinstance(ret[host], string_types) and ret[host].startswith("Minion did not return"):
                 continue
             for fun in ret[host]:
-                if fun not in docs:
-                    if ret[host][fun]:
-                        docs[fun] = ret[host][fun]
-        for fun in sorted(docs):
-            salt.output.display_output(fun + ':', 'nested', self.config)
-            print_cli(docs[fun])
-            print_cli('')
+                if fun not in docs and ret[host][fun]:
+                    docs[fun] = ret[host][fun]
+        if self.options.output:
+            for fun in sorted(docs):
+                salt.output.display_output({fun: docs[fun]}, 'nested', self.config)
+        else:
+            for fun in sorted(docs):
+                print_cli('{0}:'.format(fun))
+                print_cli(docs[fun])
+                print_cli('')
