@@ -22,6 +22,7 @@ The data structure needs to be:
 from __future__ import absolute_import, print_function
 import os
 import time
+import random
 import logging
 from datetime import datetime
 
@@ -62,18 +63,29 @@ except ImportError:
     pass
 # pylint: enable=import-error
 
+# Import tornado
+import tornado.gen  # pylint: disable=F0401
+
 log = logging.getLogger(__name__)
 
 
 def get_local_client(
         c_path=os.path.join(syspaths.CONFIG_DIR, 'master'),
         mopts=None,
-        skip_perm_errors=False):
+        skip_perm_errors=False,
+        io_loop=None,
+        auto_reconnect=False):
     '''
     .. versionadded:: 2014.7.0
 
     Read in the config and return the correct LocalClient object based on
     the configured transport
+
+    :param IOLoop io_loop: io_loop used for events.
+                           Pass in an io_loop if you want asynchronous
+                           operation for obtaining events. Eg use of
+                           set_event_handler() API. Otherwise, operation
+                           will be synchronous.
     '''
     if mopts:
         opts = mopts
@@ -86,7 +98,11 @@ def get_local_client(
         return salt.client.raet.LocalClient(mopts=opts)
     # TODO: AIO core is separate from transport
     elif opts['transport'] in ('zeromq', 'tcp'):
-        return LocalClient(mopts=opts, skip_perm_errors=skip_perm_errors)
+        return LocalClient(
+            mopts=opts,
+            skip_perm_errors=skip_perm_errors,
+            io_loop=io_loop,
+            auto_reconnect=auto_reconnect)
 
 
 class LocalClient(object):
@@ -111,7 +127,15 @@ class LocalClient(object):
     '''
     def __init__(self,
                  c_path=os.path.join(syspaths.CONFIG_DIR, 'master'),
-                 mopts=None, skip_perm_errors=False):
+                 mopts=None, skip_perm_errors=False,
+                 io_loop=None, keep_loop=False, auto_reconnect=False):
+        '''
+        :param IOLoop io_loop: io_loop used for events.
+                               Pass in an io_loop if you want asynchronous
+                               operation for obtaining events. Eg use of
+                               set_event_handler() API. Otherwise,
+                               operation will be synchronous.
+        '''
         if mopts:
             self.opts = mopts
         else:
@@ -127,12 +151,15 @@ class LocalClient(object):
         self.salt_user = salt.utils.get_specific_user()
         self.skip_perm_errors = skip_perm_errors
         self.key = self.__read_master_key()
+        self.auto_reconnect = auto_reconnect
         self.event = salt.utils.event.get_event(
                 'master',
                 self.opts['sock_dir'],
                 self.opts['transport'],
                 opts=self.opts,
-                listen=False)
+                listen=False,
+                io_loop=io_loop,
+                keep_loop=keep_loop)
         self.utils = salt.loader.utils(self.opts)
         self.functions = salt.loader.minion_mods(self.opts, utils=self.utils)
         self.returners = salt.loader.returners(self.opts, self.functions)
@@ -188,7 +215,7 @@ class LocalClient(object):
         # Looks like the timeout is invalid, use config
         return self.opts['timeout']
 
-    def gather_job_info(self, jid, tgt, tgt_type):
+    def gather_job_info(self, jid, tgt, tgt_type, **kwargs):
         '''
         Return the information about a given job
         '''
@@ -200,6 +227,7 @@ class LocalClient(object):
                                 arg=[jid],
                                 expr_form=tgt_type,
                                 timeout=timeout,
+                                **kwargs
                                )
 
         if 'jid' in pub_data:
@@ -253,6 +281,7 @@ class LocalClient(object):
             timeout=None,
             jid='',
             kwarg=None,
+            listen=False,
             **kwargs):
         '''
         Asynchronously send a command to connected minions
@@ -279,6 +308,7 @@ class LocalClient(object):
                 ret,
                 jid=jid,
                 timeout=self._get_timeout(timeout),
+                listen=listen,
                 **kwargs)
         except SaltClientError:
             # Re-raise error with specific message
@@ -352,13 +382,15 @@ class LocalClient(object):
             >>> SLC.cmd_subset('*', 'test.ping', sub=1)
             {'jerry': True}
         '''
-        group = self.cmd(tgt, 'sys.list_functions', expr_form=expr_form)
+        minion_ret = self.cmd(tgt, 'sys.list_functions', expr_form=expr_form, **kwargs)
+        minions = minion_ret.keys()
+        random.shuffle(minions)
         f_tgt = []
-        for minion, ret in six.iteritems(group):
+        for minion in minions:
+            if fun in minion_ret[minion]:
+                f_tgt.append(minion)
             if len(f_tgt) >= sub:
                 break
-            if fun in ret:
-                f_tgt.append(minion)
         func = self.cmd
         if cli:
             func = self.cmd_cli
@@ -394,7 +426,7 @@ class LocalClient(object):
 
         .. code-block:: python
 
-            >>> returns = local.cmd_batch('*', 'state.highstate', bat='10%')
+            >>> returns = local.cmd_batch('*', 'state.highstate', batch='10%')
             >>> for ret in returns:
             ...     print(ret)
             {'jerry': {...}}
@@ -525,32 +557,38 @@ class LocalClient(object):
             function name.
         '''
         arg = salt.utils.args.condition_input(arg, kwarg)
-        pub_data = self.run_job(tgt,
-                                fun,
-                                arg,
-                                expr_form,
-                                ret,
-                                timeout,
-                                jid,
-                                **kwargs)
+        was_listening = self.event.cpub
 
-        if not pub_data:
-            return pub_data
+        try:
+            pub_data = self.run_job(tgt,
+                                    fun,
+                                    arg,
+                                    expr_form,
+                                    ret,
+                                    timeout,
+                                    jid,
+                                    listen=True,
+                                    **kwargs)
 
-        ret = {}
-        for fn_ret in self.get_cli_event_returns(
-                pub_data['jid'],
-                pub_data['minions'],
-                self._get_timeout(timeout),
-                tgt,
-                expr_form,
-                **kwargs):
+            if not pub_data:
+                return pub_data
 
-            if fn_ret:
-                for mid, data in six.iteritems(fn_ret):
-                    ret[mid] = data.get('ret', {})
+            ret = {}
+            for fn_ret in self.get_cli_event_returns(
+                    pub_data['jid'],
+                    pub_data['minions'],
+                    self._get_timeout(timeout),
+                    tgt,
+                    expr_form,
+                    **kwargs):
 
-        return ret
+                if fn_ret:
+                    for mid, data in six.iteritems(fn_ret):
+                        ret[mid] = data.get('ret', {})
+            return ret
+        finally:
+            if not was_listening:
+                self.event.close_pub()
 
     def cmd_cli(
             self,
@@ -575,40 +613,51 @@ class LocalClient(object):
         :returns: A generator
         '''
         arg = salt.utils.args.condition_input(arg, kwarg)
-        pub_data = self.run_job(
-            tgt,
-            fun,
-            arg,
-            expr_form,
-            ret,
-            timeout,
-            **kwargs)
+        was_listening = self.event.cpub
 
-        if not pub_data:
-            yield pub_data
-        else:
-            try:
-                for fn_ret in self.get_cli_event_returns(
-                        pub_data['jid'],
-                        pub_data['minions'],
-                        self._get_timeout(timeout),
-                        tgt,
-                        expr_form,
-                        verbose,
-                        progress,
-                        **kwargs):
+        try:
+            self.pub_data = self.run_job(
+                tgt,
+                fun,
+                arg,
+                expr_form,
+                ret,
+                timeout,
+                listen=True,
+                **kwargs)
 
-                    if not fn_ret:
-                        continue
+            if not self.pub_data:
+                yield self.pub_data
+            else:
+                try:
+                    for fn_ret in self.get_cli_event_returns(
+                            self.pub_data['jid'],
+                            self.pub_data['minions'],
+                            self._get_timeout(timeout),
+                            tgt,
+                            expr_form,
+                            verbose,
+                            progress,
+                            **kwargs):
 
-                    yield fn_ret
-            except KeyboardInterrupt:
-                msg = ('Exiting on Ctrl-C\nThis job\'s jid is:\n{0}\n'
-                       'The minions may not have all finished running and any '
-                       'remaining minions will return upon completion. To '
-                       'look up the return data for this job later run:\n'
-                       'salt-run jobs.lookup_jid {0}').format(pub_data['jid'])
-                raise SystemExit(msg)
+                        if not fn_ret:
+                            continue
+
+                        yield fn_ret
+                except KeyboardInterrupt:
+                    raise SystemExit(
+                        '\n'
+                        'This job\'s jid is: {0}\n'
+                        'Exiting gracefully on Ctrl-c\n'
+                        'The minions may not have all finished running and any '
+                        'remaining minions will return upon completion. To look '
+                        'up the return data for this job later, run the following '
+                        'command:\n\n'
+                        'salt-run jobs.lookup_jid {0}'.format(self.pub_data['jid'])
+                    )
+        finally:
+            if not was_listening:
+                self.event.close_pub()
 
     def cmd_iter(
             self,
@@ -638,28 +687,37 @@ class LocalClient(object):
             {'stewart': {'ret': True}}
         '''
         arg = salt.utils.args.condition_input(arg, kwarg)
-        pub_data = self.run_job(
-            tgt,
-            fun,
-            arg,
-            expr_form,
-            ret,
-            timeout,
-            **kwargs)
+        was_listening = self.event.cpub
 
-        if not pub_data:
-            yield pub_data
-        else:
-            for fn_ret in self.get_iter_returns(pub_data['jid'],
-                                                pub_data['minions'],
-                                                timeout=self._get_timeout(timeout),
-                                                tgt=tgt,
-                                                tgt_type=expr_form,
-                                                **kwargs):
-                if not fn_ret:
-                    continue
-                yield fn_ret
-            self._clean_up_subscriptions(pub_data['jid'])
+        try:
+            pub_data = self.run_job(
+                tgt,
+                fun,
+                arg,
+                expr_form,
+                ret,
+                timeout,
+                listen=True,
+                **kwargs)
+
+            if not pub_data:
+                yield pub_data
+            else:
+                if kwargs.get('yield_pub_data'):
+                    yield pub_data
+                for fn_ret in self.get_iter_returns(pub_data['jid'],
+                                                    pub_data['minions'],
+                                                    timeout=self._get_timeout(timeout),
+                                                    tgt=tgt,
+                                                    tgt_type=expr_form,
+                                                    **kwargs):
+                    if not fn_ret:
+                        continue
+                    yield fn_ret
+                self._clean_up_subscriptions(pub_data['jid'])
+        finally:
+            if not was_listening:
+                self.event.close_pub()
 
     def cmd_iter_no_block(
             self,
@@ -696,31 +754,38 @@ class LocalClient(object):
             {'stewart': {'ret': True}}
         '''
         arg = salt.utils.args.condition_input(arg, kwarg)
-        pub_data = self.run_job(
-            tgt,
-            fun,
-            arg,
-            expr_form,
-            ret,
-            timeout,
-            **kwargs)
+        was_listening = self.event.cpub
 
-        if not pub_data:
-            yield pub_data
-        else:
-            for fn_ret in self.get_iter_returns(pub_data['jid'],
-                                                pub_data['minions'],
-                                                timeout=timeout,
-                                                tgt=tgt,
-                                                tgt_type=expr_form,
-                                                block=False,
-                                                **kwargs):
-                if fn_ret and any([show_jid, verbose]):
-                    for minion in fn_ret.keys():
-                        fn_ret[minion]['jid'] = pub_data['jid']
-                yield fn_ret
+        try:
+            pub_data = self.run_job(
+                tgt,
+                fun,
+                arg,
+                expr_form,
+                ret,
+                timeout,
+                listen=True,
+                **kwargs)
 
-            self._clean_up_subscriptions(pub_data['jid'])
+            if not pub_data:
+                yield pub_data
+            else:
+                for fn_ret in self.get_iter_returns(pub_data['jid'],
+                                                    pub_data['minions'],
+                                                    timeout=timeout,
+                                                    tgt=tgt,
+                                                    tgt_type=expr_form,
+                                                    block=False,
+                                                    **kwargs):
+                    if fn_ret and any([show_jid, verbose]):
+                        for minion in fn_ret.keys():
+                            fn_ret[minion]['jid'] = pub_data['jid']
+                    yield fn_ret
+
+                self._clean_up_subscriptions(pub_data['jid'])
+        finally:
+            if not was_listening:
+                self.event.close_pub()
 
     def cmd_full_return(
             self,
@@ -737,24 +802,31 @@ class LocalClient(object):
         Execute a salt command and return
         '''
         arg = salt.utils.args.condition_input(arg, kwarg)
-        pub_data = self.run_job(
-            tgt,
-            fun,
-            arg,
-            expr_form,
-            ret,
-            timeout,
-            **kwargs)
+        was_listening = self.event.cpub
 
-        if not pub_data:
-            return pub_data
+        try:
+            pub_data = self.run_job(
+                tgt,
+                fun,
+                arg,
+                expr_form,
+                ret,
+                timeout,
+                listen=True,
+                **kwargs)
 
-        return (self.get_cli_static_event_returns(pub_data['jid'],
-                                                  pub_data['minions'],
-                                                  timeout,
-                                                  tgt,
-                                                  expr_form,
-                                                  verbose))
+            if not pub_data:
+                return pub_data
+
+            return (self.get_cli_static_event_returns(pub_data['jid'],
+                                                      pub_data['minions'],
+                                                      timeout,
+                                                      tgt,
+                                                      expr_form,
+                                                      verbose))
+        finally:
+            if not was_listening:
+                self.event.close_pub()
 
     def get_cli_returns(
             self,
@@ -822,7 +894,8 @@ class LocalClient(object):
         '''
 
         while True:
-            raw = self.event.get_event(wait=0.01, tag=tag, match_type=match_type, full=True, no_block=True)
+            raw = self.event.get_event(wait=0.01, tag=tag, match_type=match_type, full=True,
+                                       no_block=True, auto_reconnect=self.auto_reconnect)
             yield raw
 
     def get_iter_returns(
@@ -938,7 +1011,7 @@ class LocalClient(object):
             # re-do the ping
             if time.time() > timeout_at and minions_running:
                 # since this is a new ping, no one has responded yet
-                jinfo = self.gather_job_info(jid, tgt, tgt_type)
+                jinfo = self.gather_job_info(jid, tgt, tgt_type, **kwargs)
                 minions_running = False
                 # if we weren't assigned any jid that means the master thinks
                 # we have nothing to send
@@ -956,7 +1029,14 @@ class LocalClient(object):
                 # if there are no more events, lets stop waiting for the jinfo
                 if raw is None:
                     break
-
+                try:
+                    if raw['data']['retcode'] > 0:
+                        log.error('saltutil returning errors on minion {0}'.format(raw['data']['id']))
+                        minions.remove(raw['data']['id'])
+                        break
+                except KeyError as exc:
+                    # This is a safe pass. We're just using the try/except to avoid having to deep-check for keys
+                    log.debug('Passing on saltutil error. This may be an error in saltclient. {0}'.format(exc))
                 # Keep track of the jid events to unsubscribe from later
                 open_jids.add(jinfo['jid'])
 
@@ -1046,7 +1126,7 @@ class LocalClient(object):
         while True:
             time_left = timeout_at - int(time.time())
             wait = max(1, time_left)
-            raw = self.event.get_event(wait, jid)
+            raw = self.event.get_event(wait, jid, auto_reconnect=self.auto_reconnect)
             if raw is not None and 'return' in raw:
                 found.add(raw['id'])
                 ret[raw['id']] = raw['return']
@@ -1135,7 +1215,7 @@ class LocalClient(object):
             data = self.returners['{0}.get_jid'.format(self.opts['master_job_cache'])](jid)
         except Exception as exc:
             raise SaltClientError('Could not examine master job cache. '
-                                  'Error occured in {0} returner. '
+                                  'Error occurred in {0} returner. '
                                   'Exception details: {1}'.format(self.opts['master_job_cache'],
                                                                   exc))
         for minion in data:
@@ -1199,7 +1279,7 @@ class LocalClient(object):
             # Wait 0 == forever, use a minimum of 1s
             wait = max(1, time_left)
             jid_tag = 'salt/job/{0}'.format(jid)
-            raw = self.event.get_event(wait, jid_tag)
+            raw = self.event.get_event(wait, jid_tag, auto_reconnect=self.auto_reconnect)
             if raw is not None and 'return' in raw:
                 if 'minions' in raw.get('data', {}):
                     minions.update(raw['data']['minions'])
@@ -1267,7 +1347,8 @@ class LocalClient(object):
                                          timeout=timeout,
                                          tgt=tgt,
                                          tgt_type=tgt_type,
-                                         expect_minions=(verbose or show_timeout)
+                                         expect_minions=(verbose or show_timeout),
+                                         **kwargs
                                          ):
             return_count = return_count + 1
             if progress:
@@ -1317,7 +1398,7 @@ class LocalClient(object):
             raise StopIteration()
         # Wait for the hosts to check in
         while True:
-            raw = self.event.get_event(timeout)
+            raw = self.event.get_event(timeout, auto_reconnect=self.auto_reconnect)
             if raw is None or time.time() > timeout_at:
                 # Timeout reached
                 break
@@ -1410,6 +1491,7 @@ class LocalClient(object):
             ret='',
             jid='',
             timeout=5,
+            listen=False,
             **kwargs):
         '''
         Take the required arguments and publish the given command.
@@ -1459,6 +1541,10 @@ class LocalClient(object):
                                                  master_uri=master_uri)
 
         try:
+            # Ensure that the event subscriber is connected.
+            # If not, we won't get a response, so error out
+            if listen and not self.event.connect_pub(timeout=timeout):
+                raise SaltReqTimeoutError()
             payload = channel.send(payload_kwargs, timeout=timeout)
         except SaltReqTimeoutError:
             raise SaltReqTimeoutError(
@@ -1490,6 +1576,102 @@ class LocalClient(object):
         return {'jid': payload['load']['jid'],
                 'minions': payload['load']['minions']}
 
+    @tornado.gen.coroutine
+    def pub_async(self,
+                  tgt,
+                  fun,
+                  arg=(),
+                  expr_form='glob',
+                  ret='',
+                  jid='',
+                  timeout=5,
+                  io_loop=None,
+                  listen=True,
+                  **kwargs):
+        '''
+        Take the required arguments and publish the given command.
+        Arguments:
+            tgt:
+                The tgt is a regex or a glob used to match up the ids on
+                the minions. Salt works by always publishing every command
+                to all of the minions and then the minions determine if
+                the command is for them based on the tgt value.
+            fun:
+                The function name to be called on the remote host(s), this
+                must be a string in the format "<modulename>.<function name>"
+            arg:
+                The arg option needs to be a tuple of arguments to pass
+                to the calling function, if left blank
+        Returns:
+            jid:
+                A string, as returned by the publisher, which is the job
+                id, this will inform the client where to get the job results
+            minions:
+                A set, the targets that the tgt passed should match.
+        '''
+        # Make sure the publisher is running by checking the unix socket
+        if (self.opts.get('ipc_mode', '') != 'tcp' and
+                not os.path.exists(os.path.join(self.opts['sock_dir'],
+                'publish_pull.ipc'))):
+            log.error(
+                'Unable to connect to the salt master publisher at '
+                '{0}'.format(self.opts['sock_dir'])
+            )
+            raise SaltClientError
+
+        payload_kwargs = self._prep_pub(
+                tgt,
+                fun,
+                arg,
+                expr_form,
+                ret,
+                jid,
+                timeout,
+                **kwargs)
+
+        master_uri = 'tcp://' + salt.utils.ip_bracket(self.opts['interface']) + \
+                     ':' + str(self.opts['ret_port'])
+        channel = salt.transport.client.AsyncReqChannel.factory(self.opts,
+                                                                io_loop=io_loop,
+                                                                crypt='clear',
+                                                                master_uri=master_uri)
+
+        try:
+            # Ensure that the event subscriber is connected.
+            # If not, we won't get a response, so error out
+            if listen and not self.event.connect_pub(timeout=timeout):
+                raise SaltReqTimeoutError()
+            payload = yield channel.send(payload_kwargs, timeout=timeout)
+        except SaltReqTimeoutError:
+            raise SaltReqTimeoutError(
+                'Salt request timed out. The master is not responding. '
+                'If this error persists after verifying the master is up, '
+                'worker_threads may need to be increased.'
+            )
+
+        if not payload:
+            # The master key could have changed out from under us! Regen
+            # and try again if the key has changed
+            key = self.__read_master_key()
+            if key == self.key:
+                raise tornado.gen.Return(payload)
+            self.key = key
+            payload_kwargs['key'] = self.key
+            payload = yield channel.send(payload_kwargs)
+
+        error = payload.pop('error', None)
+        if error is not None:
+            raise PublishError(error)
+
+        if not payload:
+            raise tornado.gen.Return(payload)
+
+        # We have the payload, let's get rid of the channel fast(GC'ed faster)
+        del channel
+
+        raise tornado.gen.Return({'jid': payload['load']['jid'],
+                                  'minions': payload['load']['minions']})
+
     def __del__(self):
         # This IS really necessary!
         # When running tests, if self.events is not destroyed, we leak 2
