@@ -9,13 +9,12 @@ minion.
 from __future__ import absolute_import
 
 # Import python libs
-import os
-import shutil
-import signal
-import logging
-import fnmatch
-import sys
 import copy
+import fnmatch
+import logging
+import os
+import signal
+import sys
 
 # Import 3rd-party libs
 # pylint: disable=import-error
@@ -26,7 +25,7 @@ try:
 except ImportError:
     HAS_ESKY = False
 # pylint: disable=no-name-in-module
-import salt.ext.six as six
+from salt.ext import six
 from salt.ext.six.moves.urllib.error import URLError
 # pylint: enable=import-error,no-name-in-module
 
@@ -38,19 +37,20 @@ except AttributeError:
 
 # Import salt libs
 import salt
-import salt.payload
-import salt.state
+import salt.config
 import salt.client
 import salt.client.ssh.client
-import salt.config
+import salt.payload
 import salt.runner
+import salt.state
+import salt.transport
 import salt.utils
 import salt.utils.args
-import salt.utils.process
-import salt.utils.minion
 import salt.utils.event
+import salt.utils.extmods
+import salt.utils.minion
+import salt.utils.process
 import salt.utils.url
-import salt.transport
 import salt.wheel
 
 HAS_PSUTIL = True
@@ -98,73 +98,7 @@ def _sync(form, saltenv=None):
         saltenv = _get_top_file_envs()
     if isinstance(saltenv, six.string_types):
         saltenv = saltenv.split(',')
-    ret = []
-    remote = set()
-    source = salt.utils.url.create('_' + form)
-    mod_dir = os.path.join(__opts__['extension_modules'], '{0}'.format(form))
-    cumask = os.umask(0o77)
-    if not os.path.isdir(mod_dir):
-        log.info('Creating module dir {0!r}'.format(mod_dir))
-        try:
-            os.makedirs(mod_dir)
-        except (IOError, OSError):
-            msg = 'Cannot create cache module directory {0}. Check permissions.'
-            log.error(msg.format(mod_dir))
-    for sub_env in saltenv:
-        log.info('Syncing {0} for environment {1!r}'.format(form, sub_env))
-        cache = []
-        log.info('Loading cache from {0}, for {1})'.format(source, sub_env))
-        # Grab only the desired files (.py, .pyx, .so)
-        cache.extend(
-            __salt__['cp.cache_dir'](
-                source, sub_env, include_pat=r'E@\.(pyx?|so|zip)$'
-            )
-        )
-        local_cache_dir = os.path.join(
-                __opts__['cachedir'],
-                'files',
-                sub_env,
-                '_{0}'.format(form)
-                )
-        log.debug('Local cache dir: {0!r}'.format(local_cache_dir))
-        for fn_ in cache:
-            relpath = os.path.relpath(fn_, local_cache_dir)
-            relname = os.path.splitext(relpath)[0].replace(os.sep, '.')
-            remote.add(relpath)
-            dest = os.path.join(mod_dir, relpath)
-            log.info('Copying {0!r} to {1!r}'.format(fn_, dest))
-            if os.path.isfile(dest):
-                # The file is present, if the sum differs replace it
-                hash_type = __opts__.get('hash_type', 'md5')
-                src_digest = salt.utils.get_hash(fn_, hash_type)
-                dst_digest = salt.utils.get_hash(dest, hash_type)
-                if src_digest != dst_digest:
-                    # The downloaded file differs, replace!
-                    shutil.copyfile(fn_, dest)
-                    ret.append('{0}.{1}'.format(form, relname))
-            else:
-                dest_dir = os.path.dirname(dest)
-                if not os.path.isdir(dest_dir):
-                    os.makedirs(dest_dir)
-                shutil.copyfile(fn_, dest)
-                ret.append('{0}.{1}'.format(form, relname))
-
-    touched = bool(ret)
-    if __opts__.get('clean_dynamic_modules', True):
-        current = set(_listdir_recursively(mod_dir))
-        for fn_ in current - remote:
-            full = os.path.join(mod_dir, fn_)
-            if os.path.isfile(full):
-                touched = True
-                os.remove(full)
-        # Cleanup empty dirs
-        while True:
-            emptydirs = _list_emptydirs(mod_dir)
-            if not emptydirs:
-                break
-            for emptydir in emptydirs:
-                touched = True
-                shutil.rmtree(emptydir, ignore_errors=True)
+    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv)
     # Dest mod_dir is touched? trigger reload if requested
     if touched:
         mod_file = os.path.join(__opts__['cachedir'], 'module_refresh')
@@ -177,27 +111,9 @@ def _sync(form, saltenv=None):
             os.remove(os.path.join(__opts__['cachedir'], 'grains.cache.p'))
         except OSError:
             log.error('Could not remove grains cache!')
-    os.umask(cumask)
     return ret
 
 
-def _listdir_recursively(rootdir):
-    file_list = []
-    for root, dirs, files in os.walk(rootdir):
-        for filename in files:
-            relpath = os.path.relpath(root, rootdir).strip('.')
-            file_list.append(os.path.join(relpath, filename))
-    return file_list
-
-
-def _list_emptydirs(rootdir):
-    emptydirs = []
-    for root, dirs, files in os.walk(rootdir):
-        if not files and not dirs:
-            emptydirs.append(root)
-    return emptydirs
-
-
 def update(version=None):
     '''
     Update the salt minion from the URL defined in opts['update_url']
@@ -214,7 +130,7 @@ def update(version=None):
 
     Returns details about the transaction upon completion.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
@@ -268,13 +184,16 @@ def sync_beacons(saltenv=None, refresh=True):
     '''
     .. versionadded:: 2015.5.1
 
-    Sync the beacons from the ``salt://_beacons`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_beacons`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    Sync beacons from ``salt://_beacons`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the beacons available to the minion.
+        If ``True``, refresh the available beacons on the minion. This refresh
+        will be performed even if no new beacons are synced. Set to ``False``
+        to prevent this refresh.
 
     CLI Example:
 
@@ -282,6 +201,7 @@ def sync_beacons(saltenv=None, refresh=True):
 
         salt '*' saltutil.sync_beacons
         salt '*' saltutil.sync_beacons saltenv=dev
+        salt '*' saltutil.sync_beacons saltenv=base,dev
     '''
     ret = _sync('beacons', saltenv)
     if refresh:
@@ -289,14 +209,15 @@ def sync_beacons(saltenv=None, refresh=True):
     return ret
 
 
-def sync_sdb(saltenv=None, refresh=False):
+def sync_sdb(saltenv=None):
     '''
-    .. versionadded:: 2015.5.7
+    .. versionadded:: 2015.5.8,2015.8.3
 
-    Sync sdb modules from the ``salt://_sdb`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_sdb`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    Sync sdb modules from ``salt://_sdb`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : False
         This argument has no affect and is included for consistency with the
@@ -308,6 +229,7 @@ def sync_sdb(saltenv=None, refresh=False):
 
         salt '*' saltutil.sync_sdb
         salt '*' saltutil.sync_sdb saltenv=dev
+        salt '*' saltutil.sync_sdb saltenv=base,dev
     '''
     ret = _sync('sdb', saltenv)
     return ret
@@ -315,13 +237,18 @@ def sync_sdb(saltenv=None, refresh=False):
 
 def sync_modules(saltenv=None, refresh=True):
     '''
-    Sync the modules from the ``salt://_modules`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_modules`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    .. versionadded:: 0.10.0
+
+    Sync execution modules from ``salt://_modules`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new execution modules are
+        synced. Set to ``False`` to prevent this refresh.
 
     .. important::
 
@@ -346,6 +273,7 @@ def sync_modules(saltenv=None, refresh=True):
 
         salt '*' saltutil.sync_modules
         salt '*' saltutil.sync_modules saltenv=dev
+        salt '*' saltutil.sync_modules saltenv=base,dev
     '''
     ret = _sync('modules', saltenv)
     if refresh:
@@ -355,20 +283,26 @@ def sync_modules(saltenv=None, refresh=True):
 
 def sync_states(saltenv=None, refresh=True):
     '''
-    Sync the states from the ``salt://_states`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_states`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    .. versionadded:: 0.10.0
+
+    Sync state modules from ``salt://_states`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available states on the minion. This refresh
+        will be performed even if no new state modules are synced. Set to
+        ``False`` to prevent this refresh.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_states
         salt '*' saltutil.sync_states saltenv=dev
+        salt '*' saltutil.sync_states saltenv=base,dev
     '''
     ret = _sync('states', saltenv)
     if refresh:
@@ -378,21 +312,27 @@ def sync_states(saltenv=None, refresh=True):
 
 def sync_grains(saltenv=None, refresh=True):
     '''
-    Sync the grains from the ``salt://_grains`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_grains`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    .. versionadded:: 0.10.0
+
+    Sync grains modules from ``salt://_grains`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion, and refresh
-        pillar data.
+        If ``True``, refresh the available execution modules and recompile
+        pillar data for the minion. This refresh will be performed even if no
+        new grains modules are synced. Set to ``False`` to prevent this
+        refresh.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_grains
         salt '*' saltutil.sync_grains saltenv=dev
+        salt '*' saltutil.sync_grains saltenv=base,dev
     '''
     ret = _sync('grains', saltenv)
     if refresh:
@@ -403,20 +343,27 @@ def sync_grains(saltenv=None, refresh=True):
 
 def sync_renderers(saltenv=None, refresh=True):
     '''
-    Sync the renderers from the ``salt://_renderers`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_renderers`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    .. versionadded:: 0.10.0
+
+    Sync renderers from ``salt://_renderers`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new renderers are synced.
+        Set to ``False`` to prevent this refresh. Set to ``False`` to prevent
+        this refresh.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_renderers
         salt '*' saltutil.sync_renderers saltenv=dev
+        salt '*' saltutil.sync_renderers saltenv=base,dev
     '''
     ret = _sync('renderers', saltenv)
     if refresh:
@@ -426,15 +373,20 @@ def sync_renderers(saltenv=None, refresh=True):
 
 def sync_returners(saltenv=None, refresh=True):
     '''
-    Sync the returners from the ``salt://_returners`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_returners`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    .. versionadded:: 0.10.0
+
+    Sync beacons from ``salt://_returners`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new returners are synced. Set
+        to ``False`` to prevent this refresh.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
@@ -451,17 +403,24 @@ def sync_proxymodules(saltenv=None, refresh=False):
     '''
     .. versionadded:: 2015.8.2
 
-    Sync the proxy modules from the ``salt://_proxy`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_proxy`` directory from that
-    environment. The default environment, if none is specified,  is ``base``.
+    Sync proxy modules from ``salt://_proxy`` to the minion
 
-    CLI Example:
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new proxy modules are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_proxymodules
         salt '*' saltutil.sync_proxymodules saltenv=dev
+        salt '*' saltutil.sync_proxymodules saltenv=base,dev
     '''
     ret = _sync('proxy', saltenv)
     if refresh:
@@ -469,22 +428,54 @@ def sync_proxymodules(saltenv=None, refresh=False):
     return ret
 
 
-def sync_output(saltenv=None, refresh=True):
+def sync_engines(saltenv=None, refresh=False):
     '''
-    Sync the output modules from the ``salt://_output`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_output`` directory from that
-    environment. The default environment, if none is specified, is ``base``.
+    .. versionadded:: 2016.3.0
+
+    Sync engine modules from ``salt://_engines`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new engine modules are synced.
+        Set to ``False`` to prevent this refresh.
 
-    CLI Example:
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_engines
+        salt '*' saltutil.sync_engines saltenv=base,dev
+    '''
+    ret = _sync('engines', saltenv)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_output(saltenv=None, refresh=True):
+    '''
+    Sync outputters from ``salt://_output`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new outputters are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_output
         salt '*' saltutil.sync_output saltenv=dev
+        salt '*' saltutil.sync_output saltenv=base,dev
     '''
     ret = _sync('output', saltenv)
     if refresh:
@@ -498,20 +489,24 @@ def sync_utils(saltenv=None, refresh=True):
     '''
     .. versionadded:: 2014.7.0
 
-    Sync utility source files from the ``salt://_utils`` directory on the Salt
-    fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_utils`` directory from that
-    environment. The default environment, if none is specified, is ``base``.
+    Sync utility modules from ``salt://_utils`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new utility modules are
+        synced. Set to ``False`` to prevent this refresh.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_utils
         salt '*' saltutil.sync_utils saltenv=dev
+        salt '*' saltutil.sync_utils saltenv=base,dev
     '''
     ret = _sync('utils', saltenv)
     if refresh:
@@ -523,18 +518,24 @@ def sync_log_handlers(saltenv=None, refresh=True):
     '''
     .. versionadded:: 2015.8.0
 
-    Sync utility source files from the ``salt://_log_handlers`` directory on
-    the Salt fileserver. This function is environment-aware, pass the desired
-    environment to grab the contents of the ``_log_handlers`` directory from
-    that environment. The default environment, if none is specified,  is
-    ``base``.
+    Sync log handlers from ``salt://_log_handlers`` to the minion
 
-    CLI Example:
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new log handlers are synced.
+        Set to ``False`` to prevent this refresh.
+
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_log_handlers
         salt '*' saltutil.sync_log_handlers saltenv=dev
+        salt '*' saltutil.sync_log_handlers saltenv=base,dev
     '''
     ret = _sync('log_handlers', saltenv)
     if refresh:
@@ -588,7 +589,9 @@ def sync_all(saltenv=None, refresh=True):
     grains, returners, output modules, renderers, and utils.
 
     refresh : True
-        Also refresh the execution modules available to the minion.
+        Also refresh the execution modules and recompile pillar data available
+        to the minion. This refresh will be performed even if no new dynamic
+        modules are synced. Set to ``False`` to prevent this refresh.
 
     .. important::
 
@@ -607,19 +610,20 @@ def sync_all(saltenv=None, refresh=True):
         See :ref:`here <reloading-modules>` for a more detailed explanation of
         why this is necessary.
 
-    CLI Example:
+    CLI Examples:
 
     .. code-block:: bash
 
         salt '*' saltutil.sync_all
         salt '*' saltutil.sync_all saltenv=dev
+        salt '*' saltutil.sync_all saltenv=base,dev
     '''
     log.debug('Syncing all')
     ret = {}
     ret['beacons'] = sync_beacons(saltenv, False)
     ret['modules'] = sync_modules(saltenv, False)
     ret['states'] = sync_states(saltenv, False)
-    ret['sdb'] = sync_sdb(saltenv, False)
+    ret['sdb'] = sync_sdb(saltenv)
     ret['grains'] = sync_grains(saltenv, False)
     ret['renderers'] = sync_renderers(saltenv, False)
     ret['returners'] = sync_returners(saltenv, False)
@@ -627,12 +631,12 @@ def sync_all(saltenv=None, refresh=True):
     ret['utils'] = sync_utils(saltenv, False)
     ret['log_handlers'] = sync_log_handlers(saltenv, False)
     ret['proxymodules'] = sync_proxymodules(saltenv, False)
+    ret['engines'] = sync_engines(saltenv, False)
     if __opts__['file_client'] == 'local':
         ret['pillar'] = sync_pillar(saltenv, False)
     if refresh:
         refresh_modules()
-        if __opts__['file_client'] == 'local':
-            refresh_pillar()
+        refresh_pillar()
     return ret
 
 
@@ -816,7 +820,8 @@ def find_job(jid):
 
 def find_cached_job(jid):
     '''
-    Return the data for a specific cached job id
+    Return the data for a specific cached job id. Note this only works if
+    cache_jobs has previously been set to True on the minion.
 
     CLI Example:
 
@@ -953,17 +958,26 @@ def revoke_auth(preserve_minion_cache=False):
 
         salt '*' saltutil.revoke_auth
     '''
-    channel = salt.transport.Channel.factory(__opts__)
-    tok = channel.auth.gen_token('salt')
-    load = {'cmd': 'revoke_auth',
-            'id': __opts__['id'],
-            'tok': tok,
-            'preserve_minion_cache': preserve_minion_cache}
+    masters = list()
+    ret = True
+    if 'master_uri_list' in __opts__:
+        for master_uri in __opts__['master_uri_list']:
+            masters.append(master_uri)
+    else:
+        masters.append(__opts__['master_uri'])
 
-    try:
-        return channel.send(load)
-    except SaltReqTimeoutError:
-        return False
+    for master in masters:
+        channel = salt.transport.Channel.factory(__opts__, master_uri=master)
+        tok = channel.auth.gen_token('salt')
+        load = {'cmd': 'revoke_auth',
+                'id': __opts__['id'],
+                'tok': tok,
+                'preserve_minion_cache': preserve_minion_cache}
+        try:
+            channel.send(load)
+        except SaltReqTimeoutError:
+            ret = False
+    return ret
 
 
 def _get_ssh_or_api_client(cfgfile, ssh=False):
@@ -977,8 +991,21 @@ def _get_ssh_or_api_client(cfgfile, ssh=False):
 def _exec(client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
     fcn_ret = {}
     seen = 0
-    for ret_comp in client.cmd_iter(
-            tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
+    if 'batch' in kwargs:
+        _cmd = client.cmd_batch
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'expr_form': expr_form,
+            'ret': ret, 'kwarg': kwarg, 'batch': kwargs['batch'],
+        }
+        del kwargs['batch']
+    else:
+        _cmd = client.cmd_iter
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'timeout': timeout,
+            'expr_form': expr_form, 'ret': ret, 'kwarg': kwarg,
+        }
+    cmd_kwargs.update(kwargs)
+    for ret_comp in _cmd(**cmd_kwargs):
         fcn_ret.update(ret_comp)
         seen += 1
         # fcn_ret can be empty, so we cannot len the whole return dict
@@ -1023,6 +1050,15 @@ def cmd(tgt,
         client = _get_ssh_or_api_client(master_cfgfile, ssh)
         fcn_ret = _exec(
             client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+
+    if 'batch' in kwargs:
+        old_ret, fcn_ret = fcn_ret, {}
+        for key, value in old_ret.items():
+            fcn_ret[key] = {
+                'out': value.get('out', 'highstate') if isinstance(value, dict) else 'highstate',
+                'ret': value,
+            }
+
     return fcn_ret
 
 
@@ -1099,7 +1135,8 @@ def runner(name, **kwargs):
 
 def wheel(name, *args, **kwargs):
     '''
-    Execute a wheel module (this function must be run on the master)
+    Execute a wheel module and function. This function must be run against a
+    minion that is local to the master.
 
     .. versionadded:: 2014.7.0
 
@@ -1119,7 +1156,17 @@ def wheel(name, *args, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' saltutil.wheel key.accept jerry
+        salt my-local-minion saltutil.wheel key.accept jerry
+        salt my-local-minion saltutil.wheel minions.connected
+
+    .. note::
+
+        Since this function must be run against a minion that is running locally
+        on the master in order to get accurate returns, if this function is run
+        against minions that are not local to the master, "empty" returns are
+        expected. The remote minion does not have access to wheel functions and
+        their return data.
+
     '''
     saltenv = kwargs.pop('__env__', 'base')
 
