@@ -9,19 +9,9 @@ Support for YUM/DNF
     <module-provider-override>`.
 
 .. note::
-    This module makes use of the **repoquery** utility, from the yum-utils_
-    package. This package will be installed as a dependency if salt is
-    installed via EPEL. However, if salt has been installed using pip, or a
-    host is being managed using salt-ssh, then as of version 2014.7.0
-    yum-utils_ will be installed automatically to satisfy this dependency.
-
     DNF is fully supported as of version 2015.5.10 and 2015.8.4 (partial
     support for DNF was initially added in 2015.8.0), and DNF is used
-    automatically in place of YUM in Fedora 22 and newer. For these versions,
-    repoquery is available from the ``dnf-plugins-core`` package.
-
-    .. _yum-utils: http://yum.baseurl.org/wiki/YumUtils
-
+    automatically in place of YUM in Fedora 22 and newer.
 '''
 
 # Import python libs
@@ -71,18 +61,18 @@ def __virtual__():
     Confine this module to yum based systems
     '''
     if __opts__.get('yum_provider') == 'yumpkg_api':
-        return False
+        return (False, "Module yumpkg: yumpkg_api provider not available")
     try:
         os_grain = __grains__['os'].lower()
         os_family = __grains__['os_family'].lower()
     except Exception:
-        return False
+        return (False, "Module yumpkg: no yum based system detected")
 
-    enabled = ('amazon', 'xcp', 'xenserver')
+    enabled = ('amazon', 'xcp', 'xenserver', 'virtuozzolinux')
 
     if os_family == 'redhat' or os_grain in enabled:
         return __virtualname__
-    return False
+    return (False, "Module yumpkg: no yum based system detected")
 
 
 def _strip_headers(output, *args):
@@ -144,82 +134,6 @@ def _yum():
     return __context__[contextkey]
 
 
-def _repoquery_pkginfo(repoquery_args):
-    '''
-    Wrapper to call repoquery and parse out all the tuples
-    '''
-    ret = []
-    for line in _repoquery(repoquery_args, ignore_stderr=True):
-        pkginfo = salt.utils.pkg.rpm.parse_pkginfo(
-            line,
-            osarch=__grains__['osarch']
-        )
-        if pkginfo is not None:
-            ret.append(pkginfo)
-    return ret
-
-
-def _check_repoquery():
-    '''
-    Check for existence of repoquery and install yum-utils if it is not
-    present.
-    '''
-    contextkey = 'yumpkg.has_repoquery'
-    if contextkey in __context__:
-        # We don't really care about the return value, we're just using this
-        # context key as a marker so that we know that repoquery is available,
-        # and we don't have to continue to repeat the check if this function is
-        # called more than once per run. If repoquery is not available, we
-        # raise an exception.
-        return
-
-    if _yum() == 'dnf':
-        # For some silly reason the core plugins and their manpages are in
-        # separate packages. The dnf-plugins-core package contains the manpages
-        # and depends on python-dnf-plugins-core (which contains the actual
-        # plugins).
-        def _check_plugins():
-            out = __salt__['cmd.run_all'](
-                ['rpm', '-q', '--queryformat', '%{VERSION}\n',
-                 'dnf-plugins-core'],
-                python_shell=False,
-                ignore_retcode=True
-            )
-            if out['retcode'] != 0:
-                return False
-            if salt.utils.compare_versions(ver1=out['stdout'],
-                                           oper='<',
-                                           ver2='0.1.15',
-                                           cmp_func=version_cmp):
-                return False
-            __context__[contextkey] = True
-            return True
-
-        if not _check_plugins():
-            __salt__['cmd.run'](
-                ['dnf', '-y', 'install', 'dnf-plugins-core >= 0.1.15'],
-                python_shell=False,
-                output_loglevel='trace'
-            )
-            # Check again now that we've installed dnf-plugins-core
-            if not _check_plugins():
-                raise CommandExecutionError('Unable to install dnf-plugins-core')
-    else:
-        if salt.utils.which('repoquery'):
-            __context__[contextkey] = True
-        else:
-            __salt__['cmd.run'](
-                ['yum', '-y', 'install', 'yum-utils'],
-                python_shell=False,
-                output_loglevel='trace'
-            )
-            # Check again now that we've installed yum-utils
-            if salt.utils.which('repoquery'):
-                __context__[contextkey] = True
-            else:
-                raise CommandExecutionError('Unable to install yum-utils')
-
-
 def _yum_pkginfo(output):
     '''
     Parse yum/dnf output (which could contain irregular line breaks if package
@@ -275,38 +189,9 @@ def _check_versionlock():
         )
 
 
-def _repoquery(repoquery_args,
-               query_format=salt.utils.pkg.rpm.QUERYFORMAT,
-               ignore_stderr=False):
-    '''
-    Runs a repoquery command and returns a list of namedtuples
-    '''
-    _check_repoquery()
-    if _yum() == 'dnf':
-        cmd = ['dnf', 'repoquery', '--quiet', '--queryformat', query_format]
-    else:
-        cmd = ['repoquery', '--plugins', '--queryformat', query_format]
-
-    cmd.extend(repoquery_args)
-    call = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
-    if call['retcode'] != 0:
-        comment = ''
-        # When checking for packages some yum modules return data via stderr
-        # that don't cause non-zero return codes. A perfect example of this is
-        # when spacewalk is installed but not yet registered. We should ignore
-        # those when getting pkginfo.
-        if 'stderr' in call and not salt.utils.is_true(ignore_stderr):
-            comment += call['stderr']
-        if 'stdout' in call:
-            comment += call['stdout']
-        raise CommandExecutionError(comment)
-    else:
-        return call['stdout'].splitlines()
-
-
 def _get_repo_options(**kwargs):
     '''
-    Returns a string of '--enablerepo' and '--disablerepo' options to be used
+    Returns a list of '--enablerepo' and '--disablerepo' options to be used
     in the yum command, based on the kwargs.
     '''
     # Get repo options from the kwargs
@@ -319,67 +204,47 @@ def _get_repo_options(**kwargs):
     if repo and not fromrepo:
         fromrepo = repo
 
-    use_dnf_repoquery = kwargs.get('repoquery', False) and _yum() == 'dnf'
     ret = []
     if fromrepo:
         log.info('Restricting to repo \'%s\'', fromrepo)
-        if use_dnf_repoquery:
-            # dnf-plugins-core renamed --repoid to --repo in version 0.1.7, but
-            # still keeps it as a hidden option for backwards compatibility.
-            # This is good, because --repo does not work at all (see
-            # https://bugzilla.redhat.com/show_bug.cgi?id=1299261 for more
-            # information). Using --repoid here so this will actually work.
-            ret.append('--repoid={0}'.format(fromrepo))
-        else:
-            ret.extend(['--disablerepo=*',
-                        '--enablerepo={0}'.format(fromrepo)])
+        ret.extend(['--disablerepo=*', '--enablerepo=' + fromrepo])
     else:
         if disablerepo:
-            if use_dnf_repoquery:
-                log.warning(
-                    'ignoring disablerepo, not supported in dnf repoquery'
-                )
-            else:
-                log.info('Disabling repo \'%s\'', disablerepo)
-                ret.append('--disablerepo={0}'.format(disablerepo))
+            targets = [disablerepo] \
+                if not isinstance(disablerepo, list) \
+                else disablerepo
+            log.info('Disabling repo(s): %s', ', '.join(targets))
+            ret.extend(
+                ['--disablerepo={0}'.format(x) for x in targets]
+            )
         if enablerepo:
-            if use_dnf_repoquery:
-                log.warning(
-                    'ignoring enablerepo, not supported in dnf repoquery'
-                )
-            else:
-                log.info('Enabling repo \'%s\'', enablerepo)
-                ret.append('--enablerepo={0}'.format(enablerepo))
+            targets = [enablerepo] \
+                if not isinstance(enablerepo, list) \
+                else enablerepo
+            log.info('Enabling repo(s): %s', ', '.join(targets))
+            ret.extend(['--enablerepo={0}'.format(x) for x in targets])
     return ret
 
 
 def _get_excludes_option(**kwargs):
     '''
-    Returns a string of '--disableexcludes' option to be used in the yum command,
+    Returns a list of '--disableexcludes' option to be used in the yum command,
     based on the kwargs.
     '''
     disable_excludes = kwargs.get('disableexcludes', '')
-
+    ret = []
     if disable_excludes:
-        if kwargs.get('repoquery', False) and _yum() == 'dnf':
-            log.warning(
-                'Ignoring disableexcludes, not supported in dnf repoquery'
-            )
-            return []
-        else:
-            log.info('Disabling excludes for \'%s\'', disable_excludes)
-            return ['--disableexcludes=\'{0}\''.format(disable_excludes)]
-    return []
+        log.info('Disabling excludes for \'%s\'', disable_excludes)
+        ret.append('--disableexcludes={0}'.format(disable_excludes))
+    return ret
 
 
 def _get_branch_option(**kwargs):
     '''
-    Returns a string of '--branch' option to be used in the yum command,
+    Returns a list of '--branch' option to be used in the yum command,
     based on the kwargs. This feature requires 'branch' plugin for YUM.
     '''
-    # Get branch option from the kwargs
     branch = kwargs.get('branch', '')
-
     ret = []
     if branch:
         log.info('Adding branch \'%s\'', branch)
@@ -421,7 +286,7 @@ def _get_yum_config():
         # fall back to parsing the config ourselves
         # Look for the config the same order yum does
         fn = None
-        paths = ('/etc/yum/yum.conf', '/etc/yum.conf')
+        paths = ('/etc/yum/yum.conf', '/etc/yum.conf', '/etc/dnf/dnf.conf')
         for path in paths:
             if os.path.exists(path):
                 fn = path
@@ -547,28 +412,6 @@ def latest_version(*names, **kwargs):
     if len(names) == 0:
         return ''
 
-    # Initialize the return dict with empty strings, and populate namearch_map.
-    # namearch_map will provide a means of distinguishing between multiple
-    # matches for the same package name, for example a target of 'glibc' on an
-    # x86_64 arch would return both x86_64 and i686 versions.
-    #
-    # Note that the logic in the for loop below would place the osarch into the
-    # map for noarch packages, but those cases are accounted for when iterating
-    # through the 'yum list' results later on. If the match for that package is
-    # a noarch, then the package is assumed to be noarch, and the namearch_map
-    # is ignored.
-    ret = {}
-    namearch_map = {}
-    for name in names:
-        ret[name] = ''
-        try:
-            arch = name.rsplit('.', 1)[-1]
-            if arch not in salt.utils.pkg.rpm.ARCHES:
-                arch = __grains__['osarch']
-        except ValueError:
-            arch = __grains__['osarch']
-        namearch_map[name] = arch
-
     repo_arg = _get_repo_options(**kwargs)
     exclude_arg = _get_excludes_option(**kwargs)
 
@@ -576,6 +419,8 @@ def latest_version(*names, **kwargs):
     if refresh:
         refresh_db(**kwargs)
 
+    cur_pkgs = list_pkgs(versions_as_list=True)
+
     # Get available versions for specified package(s)
     cmd = [_yum(), '--quiet']
     cmd.extend(repo_arg)
@@ -590,7 +435,6 @@ def latest_version(*names, **kwargs):
         if out['stderr']:
             # Check first if this is just a matter of the packages being
             # up-to-date.
-            cur_pkgs = list_pkgs()
             if not all([x in cur_pkgs for x in names]):
                 log.error(
                     'Problem encountered getting latest version for the '
@@ -607,13 +451,54 @@ def latest_version(*names, **kwargs):
             reverse=True
         )
 
+    def _check_cur(pkg):
+        if pkg.name in cur_pkgs:
+            for installed_version in cur_pkgs[pkg.name]:
+                # If any installed version is greater than the one found by
+                # yum/dnf list available, then it is not an upgrade.
+                if salt.utils.compare_versions(ver1=installed_version,
+                                               oper='>',
+                                               ver2=pkg.version,
+                                               cmp_func=version_cmp):
+                    return False
+            # pkg.version is greater than all installed versions
+            return True
+        else:
+            # Package is not installed
+            return True
+
+    ret = {}
     for name in names:
+        # Derive desired pkg arch (for arch-specific packages) based on the
+        # package name(s) passed to the function. On a 64-bit OS, "pkgame"
+        # would be assumed to match the osarch, while "pkgname.i686" would
+        # have an arch of "i686". This desired arch is then compared against
+        # the updates derived from _yum_pkginfo() above, so that we can
+        # distinguish an update for a 32-bit version of a package from its
+        # 64-bit counterpart.
+        try:
+            arch = name.rsplit('.', 1)[-1]
+            if arch not in salt.utils.pkg.rpm.ARCHES:
+                arch = __grains__['osarch']
+        except ValueError:
+            arch = __grains__['osarch']
+
+        # This loop will iterate over the updates derived by _yum_pkginfo()
+        # above, which have been sorted descendingly by version number,
+        # ensuring that the latest available version for the named package is
+        # examined first. The call to _check_cur() will ensure that a package
+        # seen by yum as "available" will only be detected as an upgrade if it
+        # has a version higher than all currently-installed versions of the
+        # package.
         for pkg in (x for x in updates if x.name == name):
-            if pkg.arch == 'noarch' or pkg.arch == namearch_map[name] \
+            # This if/or statement makes sure that we account for noarch
+            # packages as well as arch-specific packages.
+            if pkg.arch == 'noarch' or pkg.arch == arch \
                     or salt.utils.pkg.rpm.check_32(pkg.arch):
-                ret[name] = pkg.version
-                # no need to check another match, if there was one
-                break
+                if _check_cur(pkg):
+                    ret[name] = pkg.version
+                    # no need to check another match, if there was one
+                    break
         else:
             ret[name] = ''
 
@@ -880,8 +765,7 @@ def list_upgrades(refresh=True, **kwargs):
 
     The ``fromrepo``, ``enablerepo``, and ``disablerepo`` arguments are
     supported, as used in pkg states, and the ``disableexcludes`` option is
-    also supported. However, in Fedora 22 and newer all of these but
-    ``fromrepo`` is ignored.
+    also supported.
 
     .. versionadded:: 2014.7.0
         Support for the ``disableexcludes`` option
@@ -896,7 +780,7 @@ def list_upgrades(refresh=True, **kwargs):
     exclude_arg = _get_excludes_option(**kwargs)
 
     if salt.utils.is_true(refresh):
-        refresh_db(**kwargs)
+        refresh_db(check_update=False, **kwargs)
 
     cmd = [_yum(), '--quiet']
     cmd.extend(repo_arg)
@@ -911,6 +795,9 @@ def list_upgrades(refresh=True, **kwargs):
 
     return dict([(x.name, x.version) for x in _yum_pkginfo(out['stdout'])])
 
+# Preserve expected CLI usage (yum list updates)
+list_updates = salt.utils.alias_function(list_upgrades, 'list_updates')
+
 
 def info_installed(*names):
     '''
@@ -940,83 +827,6 @@ def info_installed(*names):
     return ret
 
 
-def check_db(*names, **kwargs):
-    '''
-    .. versionadded:: 0.17.0
-
-    Returns a dict containing the following information for each specified
-    package:
-
-    1. A key ``found``, which will be a boolean value denoting if a match was
-       found in the package database.
-    2. If ``found`` is ``False``, then a second key called ``suggestions`` will
-       be present, which will contain a list of possible matches.
-
-    The ``fromrepo``, ``enablerepo`` and ``disablerepo`` arguments are
-    supported, as used in pkg states, and the ``disableexcludes`` option is
-    also supported. However, in Fedora 22 and newer all of these but
-    ``fromrepo`` is ignored.
-
-    .. versionadded:: 2014.7.0
-        Support for the ``disableexcludes`` option
-
-    CLI Examples:
-
-    .. code-block:: bash
-
-        salt '*' pkg.check_db <package1> <package2> <package3>
-        salt '*' pkg.check_db <package1> <package2> <package3> fromrepo=epel-testing
-        salt '*' pkg.check_db <package1> <package2> <package3> disableexcludes=main
-    '''
-    normalize = kwargs.pop('normalize', True)
-    repo_arg = _get_repo_options(repoquery=True, **kwargs)
-    exclude_arg = _get_excludes_option(repoquery=True, **kwargs)
-
-    if _yum() == 'dnf':
-        repoquery_base = repo_arg + ['--whatprovides']
-        avail_cmd = repo_arg
-    else:
-        repoquery_base = repo_arg + exclude_arg
-        repoquery_base.extend(['--all', '--quiet', '--whatprovides'])
-        avail_cmd = repo_arg + ['--pkgnarrow=all', '--all']
-
-    if 'pkg._avail' in __context__:
-        avail = __context__['pkg._avail']
-    else:
-        # get list of available packages
-        avail = set()
-        lines = _repoquery(avail_cmd, query_format='%{NAME}_|-%{ARCH}')
-        for line in lines:
-            try:
-                name, arch = line.split('_|-')
-            except ValueError:
-                continue
-            if normalize:
-                avail.add(normalize_name('.'.join((name, arch))))
-            else:
-                avail.add('.'.join((name, arch)))
-        avail = sorted(avail)
-        __context__['pkg._avail'] = avail
-
-    ret = {}
-    # This repoquery NEEDS to be outside the loop below.  It can be slow, and running it multiple
-    # times inside the loop can degrade performance greatly.
-    if names:
-        repoquery_cmd = repoquery_base + list(names)
-        provides = sorted(
-            set(x.name for x in _repoquery_pkginfo(repoquery_cmd))
-        )
-    for name in names:
-        ret.setdefault(name, {})['found'] = name in avail
-        if not ret[name]['found']:
-            if name in provides:
-                # Package was not in avail but was found by the repoquery_cmd
-                ret[name]['found'] = True
-            else:
-                ret[name]['suggestions'] = provides
-    return ret
-
-
 def refresh_db(**kwargs):
     '''
     Check the yum repos for updated packages
@@ -1034,7 +844,7 @@ def refresh_db(**kwargs):
         Do not refresh the specified repo
 
     enablerepo
-        Refesh a disabled repo using this option
+        Refresh a disabled repo using this option
 
     branch
         Add the specified branch when refreshing
@@ -1059,6 +869,8 @@ def refresh_db(**kwargs):
         1: False,
     }
 
+    check_update_ = kwargs.pop('check_update', True)
+
     repo_arg = _get_repo_options(**kwargs)
     exclude_arg = _get_excludes_option(**kwargs)
     branch_arg = _get_branch_option(**kwargs)
@@ -1071,11 +883,13 @@ def refresh_db(**kwargs):
             update_cmd.extend(args)
 
     __salt__['cmd.run'](clean_cmd, python_shell=False)
-    result = __salt__['cmd.retcode'](update_cmd,
-                                     output_loglevel='trace',
-                                     ignore_retcode=True,
-                                     python_shell=False)
-    return retcodes.get(result, False)
+    if check_update_:
+        result = __salt__['cmd.retcode'](update_cmd,
+                                         output_loglevel='trace',
+                                         ignore_retcode=True,
+                                         python_shell=False)
+        return retcodes.get(result, False)
+    return True
 
 
 def clean_metadata(**kwargs):
@@ -1103,7 +917,7 @@ def install(name=None,
             normalize=True,
             **kwargs):
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -1387,6 +1201,8 @@ def install(name=None,
         if skip_verify:
             cmd.append('--nogpgcheck')
 
+    errors = []
+
     if targets:
         cmd = []
         if salt.utils.systemd.has_scope(__context__) \
@@ -1398,12 +1214,14 @@ def install(name=None,
         _add_common_args(cmd)
         cmd.append('install')
         cmd.extend(targets)
-        __salt__['cmd.run_all'](
+        out = __salt__['cmd.run_all'](
             cmd,
             output_loglevel='trace',
             python_shell=False,
             redirect_stderr=True
         )
+        if out['retcode'] != 0:
+            errors.append(out['stdout'])
 
     if downgrade:
         cmd = []
@@ -1414,12 +1232,14 @@ def install(name=None,
         _add_common_args(cmd)
         cmd.append('downgrade')
         cmd.extend(downgrade)
-        __salt__['cmd.run_all'](
+        out = __salt__['cmd.run_all'](
             cmd,
             output_loglevel='trace',
             python_shell=False,
             redirect_stderr=True
         )
+        if out['retcode'] != 0:
+            errors.append(out['stdout'])
 
     if to_reinstall:
         cmd = []
@@ -1430,12 +1250,14 @@ def install(name=None,
         _add_common_args(cmd)
         cmd.append('reinstall')
         cmd.extend(six.itervalues(to_reinstall))
-        __salt__['cmd.run_all'](
+        out = __salt__['cmd.run_all'](
             cmd,
             output_loglevel='trace',
             python_shell=False,
             redirect_stderr=True
         )
+        if out['retcode'] != 0:
+            errors.append(out['stdout'])
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs(versions_as_list=False)
@@ -1446,15 +1268,31 @@ def install(name=None,
         if pkgname not in ret or pkgname in old:
             ret.update({pkgname: {'old': old.get(pkgname, ''),
                                   'new': new.get(pkgname, '')}})
-    if ret:
-        __context__.pop('pkg._avail', None)
+
+    if errors:
+        raise CommandExecutionError(
+            'Error occurred installing{0} package(s)'.format(
+                '/reinstalling' if to_reinstall else ''
+            ),
+            info={'errors': errors, 'changes': ret}
+        )
+
     return ret
 
 
-def upgrade(refresh=True, skip_verify=False, **kwargs):
+def upgrade(name=None,
+            pkgs=None,
+            refresh=True,
+            skip_verify=False,
+            normalize=True,
+            **kwargs):
     '''
+    Run a full system upgrade (a ``yum upgrade`` or ``dnf upgrade``), or
+    upgrade specified packages. If the packages aren't installed, they will
+    not be installed.
+
     .. versionchanged:: 2014.7.0
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -1480,6 +1318,7 @@ def upgrade(refresh=True, skip_verify=False, **kwargs):
     .. code-block:: bash
 
         salt '*' pkg.upgrade
+        salt '*' pkg.upgrade name=openssl
 
     Repository Options:
 
@@ -1498,6 +1337,54 @@ def upgrade(refresh=True, skip_verify=False, **kwargs):
     disableexcludes
         Disable exclude from main, for a repo or for everything.
         (e.g., ``yum --disableexcludes='main'``)
+
+        .. versionadded:: 2014.7
+    name
+        The name of the package to be upgraded. Note that this parameter is
+        ignored if "pkgs" is passed.
+
+        32-bit packages can be upgraded on 64-bit systems by appending the
+        architecture designation (``.i686``, ``.i586``, etc.) to the end of the
+        package name.
+
+        Warning: if you forget 'name=' and run pkg.upgrade openssl, ALL packages
+        are upgraded. This will be addressed in next releases.
+
+        CLI Example:
+
+        .. code-block:: bash
+
+            salt '*' pkg.upgrade name=openssl
+
+        .. versionadded:: 2016.3.0
+    pkgs
+        A list of packages to upgrade from a software repository. Must be
+        passed as a python list. A specific version number can be specified
+        by using a single-element dict representing the package and its
+        version. If the package was not already installed on the system,
+        it will not be installed.
+
+        CLI Examples:
+
+        .. code-block:: bash
+
+            salt '*' pkg.upgrade pkgs='["foo", "bar"]'
+            salt '*' pkg.upgrade pkgs='["foo", {"bar": "1.2.3-4.el5"}]'
+
+        .. versionadded:: 2016.3.0
+
+    normalize : True
+        Normalize the package name by removing the architecture. This is useful
+        for poorly created packages which might include the architecture as an
+        actual part of the name such as kernel modules which match a specific
+        kernel version.
+
+        .. code-block:: bash
+
+            salt -G role:nsd pkg.upgrade gpfs.gplbin-2.6.32-279.31.1.el6.x86_64 normalize=False
+
+        .. versionadded:: 2016.3.0
+
     '''
     repo_arg = _get_repo_options(**kwargs)
     exclude_arg = _get_excludes_option(**kwargs)
@@ -1507,6 +1394,24 @@ def upgrade(refresh=True, skip_verify=False, **kwargs):
         refresh_db(**kwargs)
 
     old = list_pkgs()
+
+    targets = []
+    if name or pkgs:
+        try:
+            pkg_params = __salt__['pkg_resource.parse_targets'](
+                name=name,
+                pkgs=pkgs,
+                sources=None,
+                normalize=normalize,
+                **kwargs)[0]
+        except MinionError as exc:
+            raise CommandExecutionError(exc)
+
+        if pkg_params:
+            # Calling list.extend() on a dict will extend it using the
+            # dictionary's keys.
+            targets.extend(pkg_params)
+
     cmd = []
     if salt.utils.systemd.has_scope(__context__) \
             and __salt__['config.get']('systemd.scope', True):
@@ -1518,19 +1423,18 @@ def upgrade(refresh=True, skip_verify=False, **kwargs):
     if skip_verify:
         cmd.append('--nogpgcheck')
     cmd.append('upgrade')
+    cmd.extend(targets)
 
     __salt__['cmd.run'](cmd, output_loglevel='trace', python_shell=False)
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     ret = salt.utils.compare_dicts(old, new)
-    if ret:
-        __context__.pop('pkg._avail', None)
     return ret
 
 
 def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -1578,23 +1482,40 @@ def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     targets = [x for x in pkg_params if x in old]
     if not targets:
         return {}
+
     cmd = []
     if salt.utils.systemd.has_scope(__context__) \
             and __salt__['config.get']('systemd.scope', True):
         cmd.extend(['systemd-run', '--scope'])
     cmd.extend([_yum(), '-y', 'remove'] + targets)
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
+
+    out = __salt__['cmd.run_all'](
+        [_yum(), '-y', 'remove'] + targets,
+        output_loglevel='trace',
+        python_shell=False
+    )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
     ret = salt.utils.compare_dicts(old, new)
-    if ret:
-        __context__.pop('pkg._avail', None)
+
+    if errors:
+        raise CommandExecutionError(
+            'Error occurred removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
     return ret
 
 
 def purge(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     '''
-    .. versionchanged:: 2015.8.12,2016.3.3,Carbon
+    .. versionchanged:: 2015.8.12,2016.3.3,2016.11.0
         On minions running systemd>=205, `systemd-run(1)`_ is now used to
         isolate commands which modify installed packages from the
         ``salt-minion`` daemon's control group. This is done to keep systemd
@@ -1637,7 +1558,7 @@ def purge(name=None, pkgs=None, **kwargs):  # pylint: disable=W0613
     return remove(name=name, pkgs=pkgs)
 
 
-def hold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W0613
+def hold(name=None, pkgs=None, sources=None, normalize=True, **kwargs):  # pylint: disable=W0613
     '''
     .. versionadded:: 2014.7.0
 
@@ -1706,8 +1627,10 @@ def hold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W0613
                 ret[target]['comment'] = ('Package {0} is set to be held.'
                                           .format(target))
             else:
-                cmd = [_yum(), 'versionlock', target]
-                out = __salt__['cmd.run_all'](cmd, python_shell=False)
+                out = __salt__['cmd.run_all'](
+                    [_yum(), 'versionlock', target],
+                    python_shell=False
+                )
 
                 if out['retcode'] == 0:
                     ret[target].update(result=True)
@@ -1813,8 +1736,10 @@ def unhold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W06
                 ret[target]['comment'] = ('Package {0} is set to be unheld.'
                                           .format(target))
             else:
-                cmd = [_yum(), 'versionlock', 'delete'] + search_locks
-                out = __salt__['cmd.run_all'](cmd, python_shell=False)
+                out = __salt__['cmd.run_all'](
+                    [_yum(), 'versionlock', 'delete'] + search_locks,
+                    python_shell=False
+                )
 
                 if out['retcode'] == 0:
                     ret[target].update(result=True)
@@ -1834,7 +1759,7 @@ def unhold(name=None, pkgs=None, sources=None, **kwargs):  # pylint: disable=W06
 
 def list_holds(pattern=__HOLD_PATTERN, full=True):
     r'''
-    .. versionchanged:: Boron,2015.8.4,2015.5.10
+    .. versionchanged:: 2016.3.0,2015.8.4,2015.5.10
         Function renamed from ``pkg.get_locked_pkgs`` to ``pkg.list_holds``.
 
     List information on locked packages
@@ -1866,7 +1791,7 @@ def list_holds(pattern=__HOLD_PATTERN, full=True):
     out = __salt__['cmd.run']([_yum(), 'versionlock', 'list'],
                               python_shell=False)
     ret = []
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         match = _get_hold(line, pattern=pattern, full=full)
         if match is not None:
             ret.append(match)
@@ -1928,7 +1853,7 @@ def group_list():
         python_shell=False
     )
     key = None
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         line_lc = line.lower()
         if line_lc == 'done':
             break
@@ -1956,7 +1881,7 @@ def group_list():
 def group_info(name, expand=False):
     '''
     .. versionadded:: 2014.1.0
-    .. versionchanged:: Boron,2015.8.4,2015.5.10
+    .. versionchanged:: 2016.3.0,2015.8.4,2015.5.10
         The return data has changed. A new key ``type`` has been added to
         distinguish environment groups from package groups. Also, keys for the
         group name and group ID have been added. The ``mandatory packages``,
@@ -1967,6 +1892,16 @@ def group_info(name, expand=False):
 
     Lists packages belonging to a certain group
 
+    name
+        Name of the group to query
+
+    expand : False
+        If the specified group is an environment group, then the group will be
+        expanded and the return data will include package names instead of
+        group names.
+
+        .. versionadded:: 2016.3.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -1986,7 +1921,7 @@ def group_info(name, expand=False):
     )
 
     g_info = {}
-    for line in out.splitlines():
+    for line in salt.utils.itertools.split(out, '\n'):
         try:
             key, value = [x.strip() for x in line.split(':')]
             g_info[key.lower()] = value
@@ -2008,7 +1943,7 @@ def group_info(name, expand=False):
     pkgtypes_capturegroup = '(' + '|'.join(pkgtypes) + ')'
     for pkgtype in pkgtypes:
         target_found = False
-        for line in out.splitlines():
+        for line in salt.utils.itertools.split(out, '\n'):
             line = line.strip().lstrip(string.punctuation)
             match = re.match(
                 pkgtypes_capturegroup + r' (?:groups|packages):\s*$',
@@ -2041,12 +1976,13 @@ def group_info(name, expand=False):
 def group_diff(name):
     '''
     .. versionadded:: 2014.1.0
-    .. versionchanged:: Boron,2015.8.4,2015.5.10
+    .. versionchanged:: 2016.3.0,2015.8.4,2015.5.10
         Environment groups are now supported. The key names have been renamed,
         similar to the changes made in :py:func:`pkg.group_info
         <salt.modules.yumpkg.group_info>`.
 
-    Lists packages belonging to a certain group, and which are installed
+    Lists which of a group's packages are installed and which are not
+    installed
 
     CLI Example:
 
@@ -2329,7 +2265,7 @@ def mod_repo(repo, basedir=None, **kwargs):
         )
 
     # Build a list of keys to be deleted
-    todelete = []
+    todelete = ['disabled']
     for key in repo_opts:
         if repo_opts[key] != 0 and not repo_opts[key]:
             del repo_opts[key]
@@ -2338,6 +2274,8 @@ def mod_repo(repo, basedir=None, **kwargs):
     # convert disabled to enabled respectively from pkgrepo state
     if 'enabled' not in repo_opts:
         repo_opts['enabled'] = int(str(repo_opts.pop('disabled', False)).lower() != 'true')
+    else:
+        repo_opts.pop('disabled', False)
 
     # Add baseurl or mirrorlist to the 'todelete' list if the other was
     # specified in the repo_opts
@@ -2466,6 +2404,9 @@ def _parse_repo_file(filename):
                         'Failed to parse line in %s, offending line was '
                         '\'%s\'', filename, line.rstrip()
                     )
+                # YUM uses enabled field - create the disabled field so that comparisons works correctly in state
+                if comps[0].strip() == 'enabled':
+                    repos[repo]['disabled'] = comps[1] != "1"
 
     return (header, repos)
 
@@ -2530,9 +2471,10 @@ def owner(*paths):
     if not paths:
         return ''
     ret = {}
+    cmd_prefix = ['rpm', '-qf', '--queryformat', '%{name}']
     for path in paths:
         ret[path] = __salt__['cmd.run_stdout'](
-            ['rpm', '-qf', '--queryformat', '%{NAME}', path],
+            cmd_prefix + [path],
             output_loglevel='trace',
             python_shell=False
         )
@@ -2633,11 +2575,12 @@ def download(*packages):
         except OSError as exc:
             log.error('Unable to remove %s: %s', purge_target, exc)
 
+    cmd = ['yumdownloader', '-q', '--destdir={0}'.format(CACHE_DIR)]
+    cmd.extend(packages)
     __salt__['cmd.run'](
-        'yumdownloader -q {0} --destdir={1}'.format(
-            ' '.join(packages), CACHE_DIR
-        ),
-        output_loglevel='trace'
+        cmd,
+        output_loglevel='trace',
+        python_shell=False
     )
     ret = {}
     for dld_result in os.listdir(CACHE_DIR):
