@@ -35,6 +35,15 @@ To use the alternative configuration, append '--return_config alternative' to th
 .. code-block:: bash
 
     salt '*' test.ping --return redis --return_config alternative
+
+To override individual configuration items, append --return_kwargs '{"key:": "value"}' to the salt command.
+
+.. versionadded:: 2016.3.0
+
+.. code-block:: bash
+
+    salt '*' test.ping --return redis --return_kwargs '{"db": "another-salt"}'
+
 '''
 from __future__ import absolute_import
 
@@ -58,7 +67,8 @@ __virtualname__ = 'redis'
 
 def __virtual__():
     if not HAS_REDIS:
-        return False
+        return False, 'Could not import redis returner; ' \
+                      'redis python client is not installed.'
     return __virtualname__
 
 
@@ -93,17 +103,22 @@ def _get_serv(ret=None):
             db=db)
 
 
+def _get_ttl():
+    return __opts__['keep_jobs'] * 3600
+
+
 def returner(ret):
     '''
     Return data to a redis data store
     '''
     serv = _get_serv(ret)
-    pipe = serv.pipeline()
-    pipe.set('{0}:{1}'.format(ret['id'], ret['jid']), json.dumps(ret))
-    pipe.lpush('{0}:{1}'.format(ret['id'], ret['fun']), ret['jid'])
-    pipe.sadd('minions', ret['id'])
-    pipe.sadd('jids', ret['jid'])
-    pipe.execute()
+    pipeline = serv.pipeline(transaction=False)
+    minion, jid = ret['id'], ret['jid']
+    pipeline.hset('ret:{0}'.format(jid), minion, json.dumps(ret))
+    pipeline.expire('ret:{0}'.format(jid), _get_ttl())
+    pipeline.set('{0}:{1}'.format(minion, ret['fun']), jid)
+    pipeline.sadd('minions', minion)
+    pipeline.execute()
 
 
 def save_load(jid, load, minions=None):
@@ -111,11 +126,10 @@ def save_load(jid, load, minions=None):
     Save the load to the specified jid
     '''
     serv = _get_serv(ret=None)
-    serv.set(jid, json.dumps(load))
-    serv.sadd('jids', jid)
+    serv.setex('load:{0}'.format(jid), json.dumps(load), _get_ttl())
 
 
-def save_minions(jid, minions):  # pylint: disable=unused-argument
+def save_minions(jid, minions, syndic_id=None):  # pylint: disable=unused-argument
     '''
     Included for API consistency
     '''
@@ -127,7 +141,7 @@ def get_load(jid):
     Return the load data that marks a specified jid
     '''
     serv = _get_serv(ret=None)
-    data = serv.get(jid)
+    data = serv.get('load:{0}'.format(jid))
     if data:
         return json.loads(data)
     return {}
@@ -139,8 +153,7 @@ def get_jid(jid):
     '''
     serv = _get_serv(ret=None)
     ret = {}
-    for minion in serv.smembers('minions'):
-        data = serv.get('{0}:{1}'.format(minion, jid))
+    for minion, data in serv.hgetall('ret:{0}'.format(jid)).iteritems():
         if data:
             ret[minion] = json.loads(data)
     return ret
@@ -155,9 +168,11 @@ def get_fun(fun):
     for minion in serv.smembers('minions'):
         ind_str = '{0}:{1}'.format(minion, fun)
         try:
-            jid = serv.lindex(ind_str, 0)
+            jid = serv.get(ind_str)
         except Exception:
             continue
+        if not jid:
+            continue
         data = serv.get('{0}:{1}'.format(minion, jid))
         if data:
             ret[minion] = json.loads(data)
@@ -166,10 +181,17 @@ def get_fun(fun):
 
 def get_jids():
     '''
-    Return a list of all job ids
+    Return a dict mapping all job ids to job information
     '''
     serv = _get_serv(ret=None)
-    return list(serv.smembers('jids'))
+    ret = {}
+    for s in serv.mget(serv.keys('load:*')):
+        if s is None:
+            continue
+        load = json.loads(s)
+        jid = load['jid']
+        ret[jid] = salt.utils.jid.format_jid_instance(jid, load)
+    return ret
 
 
 def get_minions():
@@ -180,6 +202,25 @@ def get_minions():
     return list(serv.smembers('minions'))
 
 
+def clean_old_jobs():
+    '''
+    Clean out minions's return data for old jobs.
+
+    Normally, hset 'ret:<jid>' are saved with a TTL, and will eventually
+    get cleaned by redis.But for jobs with some very late minion return, the
+    corresponding hset's TTL will be refreshed to a too late timestamp, we'll
+    do manually cleaning here.
+    '''
+    serv = _get_serv(ret=None)
+    living_jids = set(serv.keys('load:*'))
+    to_remove = []
+    for ret_key in serv.keys('ret:*'):
+        load_key = ret_key.replace('ret:', 'load:', 1)
+        if load_key not in living_jids:
+            to_remove.append(ret_key)
+    serv.delete(*to_remove)
+
+
 def prep_jid(nocache=False, passed_jid=None):  # pylint: disable=unused-argument
     '''
     Do any work necessary to prepare a JID, including sending a custom id
