@@ -61,6 +61,7 @@ class SysModuleTest(integration.ModuleCase):
         allow_failure = (
                 'cp.recv',
                 'lxc.run_cmd',
+                'ipset.long_range',
                 'pkg.expand_repo_def',
                 'runtests_decorators.depends',
                 'runtests_decorators.depends_will_fallback',
@@ -75,6 +76,7 @@ class SysModuleTest(integration.ModuleCase):
                 'nspawn.restart',
                 'lowpkg.bin_pkg_info',
                 'state.apply',
+                'pip.iteritems',
         )
 
         for fun in docs:
