@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 
 # Import python libs
-import copy
+from __future__ import absolute_import
 import os
+import copy
 import tempfile
 import json
 import datetime
@@ -18,6 +19,7 @@ ensure_in_syspath('../../')
 import salt.loader
 import salt.utils
 from salt.exceptions import SaltRenderError
+from salt.ext.six.moves import builtins
 from salt.utils import get_context
 from salt.utils.jinja import (
     SaltCacheLoader,
@@ -26,6 +28,7 @@ from salt.utils.jinja import (
 )
 from salt.utils.templates import JINJA, render_jinja_tmpl
 from salt.utils.odict import OrderedDict
+from integration import TMP_CONF_DIR
 
 # Import 3rd party libs
 import yaml
@@ -167,8 +170,8 @@ class TestGetTemplate(TestCase):
         fn_ = os.path.join(TEMPLATES_DIR, 'files', 'test', 'hello_simple')
         with salt.utils.fopen(fn_) as fp_:
             out = render_jinja_tmpl(
-                    fp_.read(),
-                    dict(opts=self.local_opts, saltenv='test'))
+                fp_.read(),
+                dict(opts=self.local_opts, saltenv='test'))
         self.assertEqual(out, 'world\n')
 
     def test_fallback_noloader(self):
@@ -177,8 +180,9 @@ class TestGetTemplate(TestCase):
         if the file is not contained in the searchpath
         '''
         filename = os.path.join(TEMPLATES_DIR, 'files', 'test', 'hello_import')
-        out = render_jinja_tmpl(
-                salt.utils.fopen(filename).read(),
+        with salt.utils.fopen(filename) as fp_:
+            out = render_jinja_tmpl(
+                fp_.read(),
                 dict(opts=self.local_opts, saltenv='test'))
         self.assertEqual(out, 'Hey world !a b !\n')
 
@@ -194,8 +198,9 @@ class TestGetTemplate(TestCase):
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
         filename = os.path.join(TEMPLATES_DIR, 'files', 'test', 'hello_import')
-        out = render_jinja_tmpl(
-                salt.utils.fopen(filename).read(),
+        with salt.utils.fopen(filename) as fp_:
+            out = render_jinja_tmpl(
+                fp_.read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote',
                            'file_roots': self.local_opts['file_roots'],
                            'pillar_roots': self.local_opts['pillar_roots']},
@@ -221,12 +226,13 @@ class TestGetTemplate(TestCase):
         fc = MockFileClient()
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
-        self.assertRaisesRegexp(
-            SaltRenderError,
-            expected,
-            render_jinja_tmpl,
-            salt.utils.fopen(filename).read(),
-            dict(opts=self.local_opts, saltenv='test'))
+        with salt.utils.fopen(filename) as fp_:
+            self.assertRaisesRegexp(
+                SaltRenderError,
+                expected,
+                render_jinja_tmpl,
+                fp_.read(),
+                dict(opts=self.local_opts, saltenv='test'))
         SaltCacheLoader.file_client = _fc
 
     def test_macro_additional_log_for_undefined(self):
@@ -246,12 +252,13 @@ class TestGetTemplate(TestCase):
         fc = MockFileClient()
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
-        self.assertRaisesRegexp(
-            SaltRenderError,
-            expected,
-            render_jinja_tmpl,
-            salt.utils.fopen(filename).read(),
-            dict(opts=self.local_opts, saltenv='test'))
+        with salt.utils.fopen(filename) as fp_:
+            self.assertRaisesRegexp(
+                SaltRenderError,
+                expected,
+                render_jinja_tmpl,
+                fp_.read(),
+                dict(opts=self.local_opts, saltenv='test'))
         SaltCacheLoader.file_client = _fc
 
     def test_macro_additional_log_syntaxerror(self):
@@ -271,12 +278,13 @@ class TestGetTemplate(TestCase):
         fc = MockFileClient()
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
-        self.assertRaisesRegexp(
-            SaltRenderError,
-            expected,
-            render_jinja_tmpl,
-            salt.utils.fopen(filename).read(),
-            dict(opts=self.local_opts, saltenv='test'))
+        with salt.utils.fopen(filename) as fp_:
+            self.assertRaisesRegexp(
+                SaltRenderError,
+                expected,
+                render_jinja_tmpl,
+                fp_.read(),
+                dict(opts=self.local_opts, saltenv='test'))
         SaltCacheLoader.file_client = _fc
 
     def test_non_ascii_encoding(self):
@@ -285,8 +293,9 @@ class TestGetTemplate(TestCase):
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
         filename = os.path.join(TEMPLATES_DIR, 'files', 'test', 'hello_import')
-        out = render_jinja_tmpl(
-                salt.utils.fopen(filename).read(),
+        with salt.utils.fopen(filename) as fp_:
+            out = render_jinja_tmpl(
+                fp_.read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote',
                            'file_roots': self.local_opts['file_roots'],
                            'pillar_roots': self.local_opts['pillar_roots']},
@@ -298,8 +307,9 @@ class TestGetTemplate(TestCase):
         _fc = SaltCacheLoader.file_client
         SaltCacheLoader.file_client = lambda loader: fc
         filename = os.path.join(TEMPLATES_DIR, 'files', 'test', 'non_ascii')
-        out = render_jinja_tmpl(
-                salt.utils.fopen(filename).read(),
+        with salt.utils.fopen(filename) as fp_:
+            out = render_jinja_tmpl(
+                fp_.read(),
                 dict(opts={'cachedir': TEMPLATES_DIR, 'file_client': 'remote',
                            'file_roots': self.local_opts['file_roots'],
                            'pillar_roots': self.local_opts['pillar_roots']},
@@ -383,6 +393,8 @@ class TestGetTemplate(TestCase):
         )
 
     def test_render_with_unicode_syntax_error(self):
+        encoding = builtins.__salt_system_encoding__
+        builtins.__salt_system_encoding__ = 'utf-8'
         template = u'hello\n\n{{ bad\n\nfoo\ud55c'
         expected = r'.*---\nhello\n\n{{ bad\n\nfoo\xed\x95\x9c    <======================\n---'
         self.assertRaisesRegexp(
@@ -392,8 +404,11 @@ class TestGetTemplate(TestCase):
             template,
             dict(opts=self.local_opts, saltenv='test')
         )
+        builtins.__salt_system_encoding__ = encoding
 
     def test_render_with_utf8_syntax_error(self):
+        encoding = builtins.__salt_system_encoding__
+        builtins.__salt_system_encoding__ = 'utf-8'
         template = 'hello\n\n{{ bad\n\nfoo\xed\x95\x9c'
         expected = r'.*---\nhello\n\n{{ bad\n\nfoo\xed\x95\x9c    <======================\n---'
         self.assertRaisesRegexp(
@@ -403,6 +418,7 @@ class TestGetTemplate(TestCase):
             template,
             dict(opts=self.local_opts, saltenv='test')
         )
+        builtins.__salt_system_encoding__ = encoding
 
     def test_render_with_undefined_variable(self):
         template = "hello\n\n{{ foo }}\n\nfoo"
@@ -455,12 +471,31 @@ class TestCustomExtensions(TestCase):
             "foo": True,
             "bar": 42,
             "baz": [1, 2, 3],
-            "qux": 2.0
+            "qux": 2.0,
+            "spam": OrderedDict([
+                ('foo', OrderedDict([
+                    ('bar', 'baz'),
+                    ('qux', 42)
+                ])
+                )
+            ])
         }
         env = Environment(extensions=[SerializerExtension])
         rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
         self.assertEqual(dataset, yaml.load(rendered))
 
+    def test_serialize_yaml_str(self):
+        dataset = "str value"
+        env = Environment(extensions=[SerializerExtension])
+        rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
+        self.assertEqual(dataset, rendered)
+
+    def test_serialize_yaml_unicode(self):
+        dataset = u"str value"
+        env = Environment(extensions=[SerializerExtension])
+        rendered = env.from_string('{{ dataset|yaml }}').render(dataset=dataset)
+        self.assertEqual("!!python/unicode str value", rendered)
+
     def test_serialize_python(self):
         dataset = {
             "foo": True,
@@ -673,7 +708,8 @@ class TestDotNotationLookup(ModuleCase):
             'mocktest.ping': lambda: True,
             'mockgrains.get': lambda x: 'jerry',
         }
-        render = salt.loader.render(self.minion_opts, functions)
+        minion_opts = salt.config.minion_config(os.path.join(TMP_CONF_DIR, 'minion'))
+        render = salt.loader.render(minion_opts, functions)
         self.jinja = render.get('jinja')
 
     def render(self, tmpl_str, context=None):
